# coding=utf-8
"""
headword.py contains parameters required for redo_hw and redo_xml
"""
dictParms = {}
dictParms['ap90']= {
  'dictcode':'ap90',
  # the first group is key1, the 2nd is key2.
  #'reheadword':u'^<P>[.]{#(.*?)#}(.*?)¦',
  'reheadword':u'^<P>[.]{#(.*?)#}.*?¦',
  'firststring':r'[Page0001-a+ 46]',
  'endstring':r'[Page1179-a+ 40]',  #include this line, but no following,
  'repage':r'\[Page(.*?)]',
  'repagecol':r'^([0-9]+)-([1-3abc])',
  'comments':"""
    Page numbers are like [PagePPP.C] 
    PPP is a 3-digit page number
    C is a 1-digit column number (1,2 or 3)
  """,
  'hw1_special':{},
  'hw1_norm': [
   (r'ati-lo-romaSa','atiromaSa'),  #r preserves alphabetical
   #(r'\(.*?\)$',''),  # ending (X)
   (r'\(.*?\)',''),  # intermediate (X)
   (r' .*$',''),  # trailing space+rest
   (r'[,;.].*$',''),  # trailing comma, semicolon, period
   (r"'",''),
   (r'-.*$',''),  # -X or --X represent alternates
   (r'#}.*$',''), # another way alternates appear
   (u'¤',''),   # Not clear why this character is present
  ],
  'xml_adjust_tag':[
   (r'<>','<lb/>'),
   #(r'<>',''),
   (r'<H>','<H/>'),
   (r'<P>','<P/>'),
  ],
  'xml_add_tag':[
   (r'{%','<i>'), 
   (r'%}','</i>'),
   (r'{#','<s>'),
   (r'#}','</s>'),
   (r'{@','<b>'), # bold
   (r'@}','</b>'),
   (r'{\|',''),  # maybe, widely spaced text.
   (r'\|}',''),
  ],
  'xml_line0':(u'^.*?¦ *',''),
  'xml_all_deva':False,
  'xml_body_join':' ',
}
dictParms['skd']={
  'dictcode':'skd',
  # See logic in refine_headword for skd
  'reheadword': r'^<HI>(.*?)[, ]',
  #'reheadword': r'^<HI>(.*?)$',
  'firststring':r'[Page1-001-a+ 43]',
  'endstring':r'XXXXXXXXXX',
  'repage':r'\[Page(.*?)]',
  'repagecol':r'^([1-5]-[0-9]+)-(.*?)[+]',
  'comments':"""
    Page numbers are [Pagev-ppp-c+ n], where
    v = volume (1-5)
    ppp is 3-digit page number (pagination starts anew for each volume)
    c = column. Usu. a,b, or c indicating first, second, or third column
      Occasionally, a1,b1, or c1 or '' (no column marker). These
      appear to denote 'tabular' material, presented in three columns and
      covering entire width of page.  There are 69 such page  numbers.
    The 'end-of-volume' information is skipped (although present in the
   digitization).
  """,
  'hw1_special':{
   'pa(tra)ttraM': 'patraM',
   'durDe(rdDa)rzA': 'durDarzA'
  },
  'hw1_norm': [
   (r'Q2','Q'),  # remove nukta codes
   (r'q2','q'),
   (r' .*$',''), #change space and following characters to end of hw string
   (r'\(.*?\)',''), #Delete parenthetical expressions (alternate spellings)
   (r"'",''),
   (r';',''),
   (r'\(',''), # one case: Banja
  ],
  'xml_adjust_tag':[
   (r'<H>','<H/>'),
   (r'<>',''), # <lb/> will be introduced later
   (r'<HI>','<HI/>'),
   (r'<C([0-9]+)>',r'<C n="\1"/>'),
   (r'<P>','<P/>'),
  ],
  'xml_add_tag':[],
  'xml_all_deva':True,
  'xml_body_join':'<lb/>',
 }

