"""update.py
 Jan 2014, ejf
 This program is intended to be rather general.
 The first line of the changein file defines the separator:
 sep=X
 
 The other lines of the 'changein' file contains one change per line,
 formatted as 
old<sep>new<sep>meth
 The line is split using re.split(X,line)
 <sep> is a user supplied separator string (it is important that
    the separator string NOT be part of either the 'old' string or
    its replacement.
 The 'method' is either: 
  str:  this uses the replace method for strings
  re :  uses the re.sub  method for regular expression replacements
 Note: if 'meth' is absent, it is assumed to be 'str'.
 Jan 16, 2014 Assumes input is utf8-unicode, and similarly writes.
 Jan 17, 2014 Modify to write details of changed lines
"""

import re,sys
import codecs, unicodedata
class Change(object):
 def __init__(self,n,sep,line):
  changerec = re.split(sep,line)  # a list, so mutable
  if (len(changerec) < 3):
   out = "CHANGEIN PROBLEM at line %s\n%s\n" %(n,line)
   print out.encode('utf-8')
   exit(1)
  self.id = n-1
  self.line = line
  self.old = changerec[0]
  self.new = changerec[1]
  self.meth = changerec[2]
  if (len(changerec) > 3):
   self.comment = sep.join(changerec[3:])
  else:
   self.comment = ''
  self.count = 0 # count of number of lines changed by this transaction
  # changes is a list of line-pairs, the before and after 
  self.changepairs = [] 
  # precompile 
  if self.meth == 're':
   self.compiled = re.compile(self.old)

def init_changes(changein):
 """ determine an array of Change objects by parsing the
     input file named 'changein'. Return the array.
 """
 changes = [] # a list (so mutable)
 f = codecs.open(changein,encoding='utf-8',mode='r')
 n = 0
 sep='XXXX'
 for line in f:
  line = line.rstrip()
  n = n + 1
  if (n == 1):
   m = re.match(r'^sep=(.*)$',line)
   sep = m.group(1)
  else :
   change = Change(n,sep,line)
   changes.append(change)
 f.close()
 return changes

def apply_changes(changes,linein,nline):
 dbg=False
 line0 = linein.rstrip()
 # Pagination is represented by [PageX-Y], where X is a digit-sequence (usu.)
 # For a change such as 'e3' => 'e4', we don't want to change the Page!
 # So, we do a trick to avoid this
 line = line0
 linechanges=[] # list of changes that apply to this line
 line = re.sub('Page','PPPP',line);
 # apply all changes to a particular line
 for c in changes:
  lineprev = line
  lineold = line
  # the 'more' logic required for multiple 'contextual' replacements
  more = True
  nchk=0
  nchg=0
  while more:
   if (nchk > 10):
    out = "Problem more: \nn=%s\nline0=%s\nline=%s\nchanges=%s,old=%s" %(n,line0,line,nchanges,c.old)
    print out.encode('utf-8')
    exit(1)
   nchk = nchk + 1
   if (c.meth == 're'):
    line = re.sub(c.compiled,c.new,line)
   else: 
    line = line.replace(c.old,c.new)
   if (line == lineprev):
    more = False  #Break 'more' loop. This change has no more effect
   else:
    if dbg:  # debug
     out = "DEBUG 1a line %s change id %s %s\n" % (nline,c.id,c.line)
     print out.encode('utf-8')
     out = "DEBUG 1b old: %s\n" % lineprev
     print out.encode('utf-8')
     out = "DEBUG 1c new: %s\n" % line
     print out.encode('utf-8')
    # reset lineprev, and continue more loop
    lineprev = line
  # more loop finished. 
  # Did this change directive have any effect on this line?
  if line != lineold:
   changepair = (nline,lineold,line) # old and new
   c.changepairs.append(changepair)
   linechanges.append(c)
 # Undo the 'artificial' PPPP change
 line = re.sub('PPPP','Page',line)
 # return the last changed 'line', and the history of changes
 if dbg and (len(linechanges) > 0):
  outarr = [c.id for c in linechanges]
  out = "DEBUG: linechanges = %s\n" % outarr
  print out.encode('utf-8')
 return (line,linechanges) 

def update(filein,changein,fileout,changelog):
 # determine change structure from changein file
 changes = init_changes(changein)
 # open output
 fout = codecs.open(fileout,'w','utf-8')
 # open input
 f = codecs.open(filein,encoding='utf-8',mode='r')
 n = 0
 # apply all changes to each line of filein.
 all_linechanges = [] # 
 for linein in f:
  n = n+1
  (newline,linechanges) = apply_changes(changes,linein,n)
  fout.write("%s\n" % newline)
  if len(linechanges) > 0:
   all_linechanges.append((n,linein,newline,linechanges))
 # close
 f.close()
 fout.close()
 mchanges = len(changes)
 changecounts = [0 for ic in xrange(0,mchanges)]
 # write summary of changes performed
 # open changelog
 fchg = codecs.open(changelog,'w','utf-8')
 for (n,linein,line,linechanges) in all_linechanges:
  outarr=[]
  kchanges = len(linechanges)
  out = "line# %s underwent %s change(s)\n" %(n,kchanges)
  outarr.append(out)
  out = "old: %s\n" % linein
  outarr.append(out)
  out = "new: %s\n" % line
  outarr.append(out)
  # Add a list of the change ids applied to this line
  cids = [("cid#%s" % c.id) for c in linechanges]
  cidstr = ','.join(cids)
  out = "ids of changes applied: %s\n" % cidstr
  outarr.append(out)
  out = ('-'*50) + "\n"
  outarr.append(out)
  # write outarr to changelog
  fchg.write(''.join(outarr))
  # update counts    
  for c in linechanges:
   c.count = c.count + 1
 # write summary by change to changelog
 out = "\n\nSummary of changes\n";
 fchg.write(out)
 totcount = 0
 unused = []
 for c in changes:
  #(old,new,meth,count,compiled) = changerec
  out = "change cid#%s %4d  =  %s\n" % (c.id,c.count,c.line)
  #
  fchg.write(out)
  totcount = totcount + c.count
  if c.count == 0:
   unused.append(c.id)
  #print "old=%s, new=%s, meth=%s, count=%s" %(old,new,meth,count)
 out = "%s lines changed\n" % len(all_linechanges)
 print out.encode('utf-8')
 fchg.write(out)
 out = "%s changes made\n" % totcount
 print out.encode('utf-8')
 fchg.write(out)
 if len(unused) == 0:
  out = "All changes were used\n"
 else:
  out = "WARNING: %s unused changes (by id):\n" %(len(unused),','.join(unused))
 print out.encode('utf-8')
 fchg.write(out)
 fchg.close()
if __name__=="__main__":
 filein = sys.argv[1]
 changein = sys.argv[2]
 fileout = sys.argv[3]
 changelog = sys.argv[4]
 update(filein,changein,fileout,changelog)

